
<div id='id-transfer-div' class="panel panel-info">
	<div class="panel-heading" class='clearfix'>
 		 Transactions from USB. . <div id="id-div-resource-alert"></div><span id="id-tx-spinner" class="fas fa-refresh pull-right"></span>
	<div class="clearfix"></div>
	</div>
	<div class="panel-body">
		<p class='clearfix'>
			<span id="">Transactions from USB.</span>
			<span id="id-pending-transactions-middleman" class="badge"></span>
			<button id="btn-usb-upload" type="button" class="btn btn-default pull-right">Upload</button>
		</p>
		<dl id="usb-files-ul">
	  </dl> 
		<p>
		<ul  class='list-group'>
		</ul>
	</div>
 </div>
 {% if upload_transaction_files %}
 <div id='id-upload-transaction-div' class="panel panel-default">
	<div class='alert alert-warning' id="progress-status-div-pending">
 		Transactions History
	</div>
	<table class="table table-bordered" id='id-table-upload-transaction-table'>
	    <thead>
	      <tr>
	      	<th></th>
	        <th>Filename</th>
	        <th>Date</th>
	        <th>Consumed</th>
	        <th>Not Consumed</th>
	        <th>Comment</th>
	        <th>Producer</th>
	      </tr>
	    </thead>
	    <tbody>
		    {% for file in upload_transaction_files %}
		    	<tr>
		    		<td>
		    		</td>
			    	<td>
			    		{{file.filename}}
			    	</td>
			    	<td>
			    		{{file.file_date}}
			    	</td>
			    	<td>
		 				{{file.consumed}}
			    	</td>
			    	<td>
		 				{{file.not_consumed}}
			    	</td>
			    	<td>
		 				{{file.producer}}
			    	</td>
			    	<td>
		 				{{file.comment}}
			    	</td>
		    	</tr>
		    {% endfor %}
	    </tbody>
	  </table>
 </div>
 {% endif %}
 